<?php

namespace ACL\Controllers;

use ACL\Traits\TResponse;
use Exception;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use ACL\Services\Operador as OperadorService;

class Operador
{

    use TResponse;

    /**
     * @var OperadorService
     */
    private $operadorService;

    public function __construct(OperadorService $operadorService)
    {
        $this->operadorService = $operadorService;
    }

    /**
     * @param Request $request
     * @param Response $response
     * @throws Exception
     */
    public function findAllOperador(Request $request, Response $response)
    {
        try {
            $params = $request->getParameters();
            $operators = $this->operadorService->findAllOperador($params);
            $this->sendResponse($operators, 200);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function createOperador(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->operadorService->createOperador($params);
            $this->sendResponse([], 201);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function updateOperador(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->operadorService->updateOperador($params);
            $this->sendResponse([], 200);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function findOperadoresByPerfilAcesso(Request $request, Response $response)
    {
        try{
            $nrperfilacesso = $request->getParameter('NRPERFILACESSO');
            $data = $this->operadorService->findOperadoresByPerfilAcesso($nrperfilacesso);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findOperadoresForAddPerfilAcesso(Request $request, Response $response)
    {
        try{
            $nrperfilacesso = $request->getParameter('NRPERFILACESSO');
            $data = $this->operadorService->findOperadoresForAddPerfilAcesso($nrperfilacesso);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findOrganizacaoRelacionadaByOperador(Request $request, Response $response)
    {
        try{
            $params = $request->getParameters();
            $search = isset($params['search']) ? $params['search'] : null;
            $isPrimaryOrg = false;
            $data = $this->operadorService->findOrganizacaoRelacionadaByOperador(null, $isPrimaryOrg, null, $search);
            $this->sendResponse($this->createResponse($data), 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
