<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Traits\TCrud;
use Exception;

class PerfilacessoLabel
{

    use TCrud;

    /**
     * @var int
     */
    public $majorVersionInFile = 0;

    /**
     * @var int
     */
    public $oldMajorVersion = 0;

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;
    /**
     * @var \ACL\Repositories\PerfilacessoLabel
     */
    private $perfilacessoLabelRepository;

    /**
     * @var array
     */
    private $nodes = [];

    public function __construct()
    {
        $this->libEnvironment = InstanceProvider::getLibEnvironment();
        $this->perfilacessoLabelRepository = InstanceProvider::getRepository()->getPerfilacessoLabel();
    }

    /**
     * @param $nrperfilacesso
     * @param $oldMajorVersion
     * @param bool $versionSave
     * @return mixed
     * @throws \Doctrine\DBAL\Exception
     * @throws Exception
     * @throws \Doctrine\DBAL\Driver\Exception
     */
   public function findPerfilacessoLabel($nrperfilacesso, $oldMajorVersion, bool $versionSave = false)
   {
       $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacesso($nrperfilacesso, getNrorg());
       $this->nodes = $this->getFileAclLabelsJson();

       if (!$this->nodes) {
           throw new Exception('Arquivo "aclLabels.json" não encontrado');
       }

       $this->oldMajorVersion = $oldMajorVersion;

       if ($versionSave) {
           $this->setMajorVersionInFile($this->nodes);
           InstanceProvider::getService()->getOperadorLabelVersao()->save($this->majorVersionInFile);
       }

       return $this->setLabelsVisibility($perfilacessoLabels, $this->nodes);
   }

    /**
     * @return mixed
     * @throws Exception
     */
   private function getFileAclLabelsJson()
   {
       $productBasePath = InstanceProvider::getLibUtilities()->getProductBasePath();
       return json_decode(@file_get_contents($productBasePath . 'aclLabels.json'), true);
   }

    /**
     * @param $perfilacessoLabels
     * @param $labelsFile
     * @return mixed
     * @throws Exception
     */
   private function setLabelsVisibility($perfilacessoLabels, &$labelsFile)
   {
       foreach ($labelsFile as &$label) {
           if (!empty($label['data']['version']) && $label['data']['version'] > $this->oldMajorVersion) {
               $label['data']['new'] = true;
               if (!empty($label['data']['parent'])) {
                   $this->setParentNodeIsNew($this->nodes, $label['data']['parent']);
               }
           }

           foreach ($perfilacessoLabels as $row) {
               if ($label['data']['type'] !== 'GROUPED' && $row['LABEL'] == $label['data']['label']) {
                   $label['data']['checked'] = true;
               }
           }

           if ($this->hasChildren($label)) {
               $this->setLabelsVisibility($perfilacessoLabels, $label['children']);
           }
       }
       return $labelsFile;
   }

   private function setParentNodeIsNew(&$labelsFile, $parent)
   {
       foreach ($labelsFile as &$label) {
           if ($label['data']['type'] !== 'GROUPED' && $label['data']['label'] === $parent) {
               $label['data']['new'] = true;
               if(!empty($label['data']['parent'])) {
                   $this->setParentNodeIsNew($this->nodes, $label['data']['parent']);
               }
           } else if ($label['data']['type'] === 'GROUPED' && $label['title'] === $parent){
               $label['data']['new'] = true;
           }

           if($this->hasChildren($label)) {
               $this->setParentNodeIsNew($label['children'], $parent);
           }
       }
   }

    /**
     * @param $node
     * @return bool
     */
   private function hasChildren($node): bool
   {
       return array_key_exists('children', $node) && is_array($node['children']) && count($node['children']);
   }

    /**
     * @return int
     * @throws Exception
     */
    private function getMajorVersionInBD(): int
    {
        $data = InstanceProvider::getService()->getOperadorLabelVersao()->findByOperadorAndProject();
        if (empty($data)) {
            return 1;
        }
        return intval($data['NRULTVERSAO']);
    }

    /**
     * @param $labelsFile
     */
    public function setMajorVersionInFile($labelsFile)
    {
        foreach ($labelsFile as $label) {
            if ($label['data']['version'] > $this->majorVersionInFile) {
                $this->majorVersionInFile = $label['data']['version'];
            }
            $this->setMajorVersionInFile($label['children']);
        }
    }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function savePerfilacessoLabel($labels, $nrperfilacesso)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacessoAndLabels($nrperfilacesso, $labelsQueryIn, getNrorg());

        if (empty($perfilacessoLabels)) {
            foreach ($labels as $label) {
                $this->createPerfilacessoLabel($nrperfilacesso, $label);
            }
        } else {
            foreach ($labels as $label) {
                $key = in_array($label, array_column($perfilacessoLabels, 'LABEL'));
                if ($key === false) {
                    $this->createPerfilacessoLabel($nrperfilacesso, $label);
                }
            }
        }
    }

    /**
     * @param array $labels
     * @param string $column
     * @param string $condition
     * @return string
     */
    private function getStringInQueryByArray(array $labels, string $column, string $condition = 'AND'): string
    {
        $queryIn = "$condition $column IN (";
        foreach ($labels as $label) {
            $queryIn .= "'" . $label . "', ";
        }
        $queryIn = substr($queryIn, 0, -2) . ")";
        return $queryIn;
    }

    /**
     * @param $nrperfilacesso
     * @param $label
     * @param string $idativo
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    private function createPerfilacessoLabel($nrperfilacesso, $label, $idativo = 'S')
    {
        $newCode = InstanceProvider::getNewCode()->getCodeByTableName('PERFILACESSO_LABEL', getNrorg(), 1, 10, true);
        $auditFields = $this->perfilacessoLabelRepository->getAuditFieldValuesDefaultInsert();
        $this->perfilacessoLabelRepository->savePerfilacessoLabel(
            $newCode, $label, intval($nrperfilacesso), $auditFields['NRORG'], $idativo, $auditFields['CDOPERINCLUSAO']);
    }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function deletePerfilacessoLabel($labels, $nrperfilacesso)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $this->perfilacessoLabelRepository->deletePerfilacessoLabel($nrperfilacesso, $labelsQueryIn, getNrorg());
    }
}
