/*
@C:\WEB_619\SQL\END_CONV\SQLSERVER\5205_DFESTPS_IMPOSTO_ENTRADA_PRODUTO.SQL

 Descrio:
 Procedure para gerao dos impostos por produto

 Criao:
 07/2012 - Lucas Primo dos Santos

*/

IF EXISTS (SELECT *
             FROM SYSOBJECTS O, SYSUSERS U
            WHERE O.NAME = 'IMPOSTO_ENTRADA_PRODUTO'
              AND TYPE = 'P'
              AND U.UID = O.UID
              AND U.NAME = USER)

DROP PROCEDURE IMPOSTO_ENTRADA_PRODUTO
GO

CREATE PROCEDURE IMPOSTO_ENTRADA_PRODUTO
   @P_EMPRESA      VARCHAR(02),                    --EMPRESA
   @P_INSCESTA     VARCHAR(16),                    --INSCRIO ESTADUAL
   @P_TIPOOPER     VARCHAR(02),                    --TIPO DE OPERAO
   @P_CDFORNECED   VARCHAR(21),                    --CDIGO FORNECEDOR
   @P_FILIAL       VARCHAR(04),                    --FILIAL
   @P_LANCTONF     VARCHAR(10),                    --N LANCAMENTO
   @P_INIVALID     DATETIME,                       --DATA DE INICIO DE VALIDADDE
   @P_ENTRSAID     VARCHAR(01) = 'E',              --PARMETRO P/ IMPORTACAO DE NOTAS DE SADA
   @P_CDOPERADOR   VARCHAR(12) = NULL,
   @P_CDGRUPOFISC  VARCHAR(04),
   @P_IDINTEREST   VARCHAR(01),
   @P_CDARVPROD    VARCHAR(20),
   @P_CDGRPFISCSV  VARCHAR(20),
   @P_IDORIGDESTNF VARCHAR(01),
   @P_CDFILIRECE   VARCHAR(04),
   @P_CDFILIEMIT   VARCHAR(04),
   @P_CDCLIENTE    VARCHAR(21),
   @P_VRPRODUTO    NUMERIC(15,6),
   @P_IDORIGEMPROD VARCHAR(01) = 'N'

AS
   DECLARE @V_IDOPERDEVONF  VARCHAR(01);
   DECLARE @V_IDORIGDESTNF  VARCHAR(01);
   DECLARE @V_DEVOLORIG     VARCHAR(10);
   DECLARE @V_EXISTE        NUMERIC(15,6);
   DECLARE @V_PAISORIG      VARCHAR(04);
   DECLARE @V_ESTAORIG      VARCHAR(02);
   DECLARE @V_MUNIORIG      VARCHAR(04);
   DECLARE @V_PAISDEST      VARCHAR(04);
   DECLARE @V_ESTADEST      VARCHAR(02);
   DECLARE @V_MUNIDEST      VARCHAR(04);
   DECLARE @V_ALIQTEMP      VARCHAR(25);
   DECLARE @V_INCITEMP      VARCHAR(25);
   DECLARE @V_CDORIGDESTNF  VARCHAR(25);
   DECLARE @V_VRALIQUTMP    DOUBLE PRECISION;

   /* VARIVEIS DO CURSOR C_INCIDENCIA*/
   DECLARE @V_INCIDENC_CDIMPOTEMP   VARCHAR(02);
   DECLARE @V_INCIDENC_CDFISCTEMP   VARCHAR(01);
   DECLARE @V_INCIDENC_IDTRATIMPTMP VARCHAR(01);
   DECLARE @V_INCIDENC_IDCONSISTENT VARCHAR(01);
   DECLARE @V_INCIDENC_IDIMPORECUP  VARCHAR(01);
   DECLARE @V_INCIDENC_IDPREIMPENTR VARCHAR(01);
   DECLARE @V_INCIDENC_IDUTMUNIORIG VARCHAR(01);
   DECLARE @V_INCIDENC_IDORGARRECAD VARCHAR(01);
   DECLARE @V_INCIDENC_IDUTVLRQTDEIMPO VARCHAR(01);
   DECLARE @V_IDNFPRESTSERV         VARCHAR(01);

   DECLARE @V_IDENDEMIS      VARCHAR(01);
   DECLARE @V_CDEMPRESA      VARCHAR(02);
   DECLARE @V_CDINSCESTA     VARCHAR(16);

   DECLARE @V_IDENTEMIS      VARCHAR(01);
BEGIN
   /*VARIVEIS COM A CHAVE PARA AS TABELAS TEMPORRIAS*/

   SET @V_ALIQTEMP     = @P_FILIAL + @P_LANCTONF;
   SET @V_INCITEMP     = @P_FILIAL + @P_LANCTONF;
   SET @V_IDORIGDESTNF = @P_IDORIGDESTNF;

   /*CASO TENHA REGISTRO NA TABELA TEMPORRIA, OS REGISTROS SO EXCLUDOS*/
   SELECT @V_EXISTE = COUNT(*)
     FROM INCITEMP
    WHERE NRINCITEMP = @P_FILIAL + @P_LANCTONF;

   IF @V_EXISTE > 0 BEGIN
      DELETE FROM INCITEMP
       WHERE NRINCITEMP = @P_FILIAL + @P_LANCTONF;
   END;

   /*VARIVEIS PARA CONSULTA DE ALQUOTA, INCIDNCIA */
   SELECT @V_PAISDEST = MAX(CDPAIS),
          @V_ESTADEST = MAX(SGESTADO),
          @V_MUNIDEST = MAX(CDMUNICIPIO)
     FROM INSCRESTAD
    WHERE (CDEMPRESA  = @P_EMPRESA)
      AND (CDINSCESTA = @P_INSCESTA);

   SELECT @V_IDNFPRESTSERV = ISNULL(MAX(IDNFPRESTSERV), 'N')
     FROM TIPOOPERAC
    WHERE IDENTRSAIDOP = 'E'
      AND CDTIPOOPER   = @P_TIPOOPER;

   IF @V_IDNFPRESTSERV = 'S' BEGIN

      SELECT @V_IDENTEMIS = ISNULL(MAX(IDENDEMISFIL),'P')
        FROM FILIAL
       WHERE (CDFILIAL = @P_CDFILIRECE);

      IF @V_IDENTEMIS = 'I' BEGIN
         SET @V_IDENTEMIS = 'P';
      END;

      SELECT @V_PAISDEST = MAX(CDPAIS),
             @V_ESTADEST = MAX(SGESTADO),
             @V_MUNIDEST = MAX(CDMUNICIPIO)
        FROM ENDEFILI
       WHERE (CDFILIAL     = @P_CDFILIRECE)
         AND (IDTPENDEFILI = @V_IDENTEMIS);
   END;

   SET @V_CDORIGDESTNF = @P_CDFORNECED;

   /*SE O IDENTIFICADOR DE ORIGEM DO LANAMENTO FOR IGUAL A FILIAL*/
   IF @V_IDORIGDESTNF = 'L' BEGIN
      SET @V_CDORIGDESTNF = @P_CDFILIEMIT;

      SET @V_IDENDEMIS  = 'I';
      SET @V_CDEMPRESA  = Null;
      SET @V_CDINSCESTA = Null;

      SELECT @V_IDENDEMIS = ISNULL(MAX(IDENDEMISFIL),'I'), @V_CDEMPRESA = MAX(CDEMPRESA), @V_CDINSCESTA = MAX(CDINSCESTA)
      FROM FILIAL
      WHERE (CDFILIAL = @P_CDFILIEMIT);

      IF @V_IDENDEMIS = 'I' BEGIN

         SET @V_PAISORIG = Null;
         SET @V_ESTAORIG = Null;
         SET @V_MUNIORIG = Null;

         SELECT @V_PAISORIG = MAX(CDPAIS), @V_ESTAORIG = MAX(SGESTADO), @V_MUNIORIG = MAX(CDMUNICIPIO)
         FROM INSCRESTAD
         WHERE CDEMPRESA  = @V_CDEMPRESA AND
               CDINSCESTA = @V_CDINSCESTA;
      END
      ELSE BEGIN
         SET @V_PAISORIG = Null;
         SET @V_ESTAORIG = Null;
         SET @V_MUNIORIG = Null;

         SELECT @V_PAISORIG = MAX(CDPAIS), @V_ESTAORIG = MAX(SGESTADO), @V_MUNIORIG = MAX(CDMUNICIPIO)
         FROM ENDEFILI
         WHERE CDFILIAL     = @P_CDFILIEMIT AND
               IDTPENDEFILI = @V_IDENDEMIS;
      END;

   /*SE O IDENTIFICADOR DE ORIGEM DO LANAMENTO FOR IGUAL A CLIENTE*/
   END
   ELSE IF @V_IDORIGDESTNF = 'C' BEGIN
      SELECT @V_PAISORIG = MAX(CDPAIS),
             @V_ESTAORIG = MAX(SGESTADO),
             @V_MUNIORIG = MAX(CDMUNICIPIO)
        FROM ENDECLIE
       WHERE (CDCLIENTE    = @P_CDCLIENTE) AND
             (IDTPENDECLIE = 'P');
   /*SE O IDENTIFICADOR DE ORIGEM DO LANAMENTO FOR IGUAL A FORNECEDOR*/
   END
   ELSE IF @V_IDORIGDESTNF = 'F' BEGIN
      SELECT @V_PAISORIG = MAX(CDPAIS),
             @V_ESTAORIG = MAX(SGESTADO),
             @V_MUNIORIG = MAX(CDMUNICIPIO)
        FROM ENDEFORN
       WHERE (CDFORNECED   = @P_CDFORNECED) AND
             (IDTPENDEFORN = 'P');
   END
   ELSE BEGIN
      /*SE O IDENTIFICADOR DE ORIGEM DO LANAMENTO FOR IGUAL A LICENCA DE USO OU OUTRO VALOR DIFERENTE DE L,C,F */
      SELECT @V_PAISORIG = MAX(CDPAIS),
             @V_ESTAORIG = MAX(SGESTADO),
             @V_MUNIORIG = MAX(CDMUNICIPIO)
        FROM ENDEFORN
       WHERE (CDFORNECED   = @P_CDFORNECED) AND
             (IDTPENDEFORN = 'P');
   END;

   IF(NOT @P_TIPOOPER IN ('07','09','10'))  BEGIN

      EXEC CONSULTA_INCIDENCIA @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @P_TIPOOPER, @P_FILIAL, @P_LANCTONF, @P_CDGRUPOFISC, @P_CDGRPFISCSV, @P_IDINTEREST, '', @P_CDFILIRECE, @P_CDFORNECED, @P_CDCLIENTE, @P_INIVALID, @P_CDFILIEMIT, @P_VRPRODUTO, 'S', @P_IDORIGEMPROD;

      DECLARE C_INCIDENCIA CURSOR FOR
      SELECT IT.CDIMPOTEMP, MAX(IT.CDFISCTEMP) AS CDFISCTEMP,
             MAX(IT.IDTRATIMPTMP) AS IDTRATIMPTMP, MAX(IP.IDCONSISTENT) AS IDCONSISTENT,
             MAX(ISNULL(IP.IDUTMUNIORIG, 'N')) AS IDUTMUNIORIG, MAX(IP.IDIMPORECUP) AS IDIMPORECUP,
             MAX(ISNULL(IP.IDPREIMPENTR, 'N')) AS IDPREIMPENTR,MAX(IP.IDORGARRECAD) AS IDORGARRECAD,
             MAX(IDUTVLRQTDEIMPO) AS IDUTVLRQTDEIMPO
        FROM INCITEMP IT, IMPOSTO IP
       WHERE (IT.NRINCITEMP = @V_INCITEMP)
         AND (IP.CDIMPOSTO  = IT.CDIMPOTEMP)
         AND (IP.IDPRODNOTA = 'P')
       GROUP BY IT.CDIMPOTEMP
       ORDER BY IT.CDIMPOTEMP;

       OPEN C_INCIDENCIA;
       FETCH NEXT FROM C_INCIDENCIA INTO @V_INCIDENC_CDIMPOTEMP,   @V_INCIDENC_CDFISCTEMP,
                                         @V_INCIDENC_IDTRATIMPTMP, @V_INCIDENC_IDCONSISTENT,
                                         @V_INCIDENC_IDUTMUNIORIG, @V_INCIDENC_IDIMPORECUP,
                                         @V_INCIDENC_IDPREIMPENTR, @V_INCIDENC_IDORGARRECAD,
                                         @V_INCIDENC_IDUTVLRQTDEIMPO;
      WHILE (@@FETCH_STATUS = 0) BEGIN

         IF @V_INCIDENC_IDUTMUNIORIG = 'D' BEGIN
            EXEC CONSULTA_ALIQUOTA @V_INCIDENC_CDIMPOTEMP, @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @V_PAISORIG,
                                   @V_ESTAORIG, @V_PAISDEST, @V_ESTADEST, @V_MUNIDEST, @P_VRPRODUTO, @P_INIVALID, @P_FILIAL,
                                   @P_LANCTONF, @P_CDGRUPOFISC, @P_TIPOOPER, '', 'S', @P_VRPRODUTO,
                                   @P_CDFILIRECE, @P_CDFORNECED, @P_CDCLIENTE, @P_IDORIGEMPROD;

         END
         ELSE BEGIN
            EXEC CONSULTA_ALIQUOTA @V_INCIDENC_CDIMPOTEMP, @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @V_PAISORIG,
                                   @V_ESTAORIG, @V_PAISDEST, @V_ESTADEST, @V_MUNIORIG, @P_VRPRODUTO, @P_INIVALID, @P_FILIAL,
                                   @P_LANCTONF, @P_CDGRUPOFISC, @P_TIPOOPER, '', 'S', @P_VRPRODUTO,
                                   @P_CDFILIRECE, @P_CDFORNECED, @P_CDCLIENTE, @P_IDORIGEMPROD;

         END;

         SET @V_VRALIQUTMP = 0;

         SELECT @V_VRALIQUTMP = ISNULL(MAX(VRALIQUTMP), 0)
           FROM ALIQTEMP
          WHERE NRALIQTEMP = @V_ALIQTEMP;

         INSERT
           INTO ALIQTEMPPROD
                (NRALIQTEMP, CDIMPOTEMP, VRALIQUTMP)
         VALUES (@V_ALIQTEMP, @V_INCIDENC_CDIMPOTEMP, @V_VRALIQUTMP);

         /*CASO TENHA REGISTRO NA TABELA TEMPORRIA, OS REGISTROS SO EXCLUDOS*/
         SELECT @V_EXISTE = COUNT(*)
           FROM ALIQTEMP
          WHERE NRALIQTEMP = @P_FILIAL + @P_LANCTONF;

         IF @V_EXISTE > 0 BEGIN
            DELETE
              FROM ALIQTEMP
             WHERE NRALIQTEMP = @P_FILIAL + @P_LANCTONF;
         END;


         FETCH NEXT FROM C_INCIDENCIA INTO @V_INCIDENC_CDIMPOTEMP,   @V_INCIDENC_CDFISCTEMP,
                                           @V_INCIDENC_IDTRATIMPTMP, @V_INCIDENC_IDCONSISTENT,
                                           @V_INCIDENC_IDUTMUNIORIG, @V_INCIDENC_IDIMPORECUP,
                                           @V_INCIDENC_IDPREIMPENTR, @V_INCIDENC_IDORGARRECAD,
                                           @V_INCIDENC_IDUTVLRQTDEIMPO;





      END; /* WHILE DO CURSOR C_INCIDENCIA*/

      CLOSE C_INCIDENCIA;
      DEALLOCATE C_INCIDENCIA;

   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'IMPOSTO_ENTRADA_PRODUTO','6.19.XXX';
GO


